@@ -19,7 +19,7 @@ class PyXtal_FF():
                 The type of atom-centered descriptors.
                 + ACSF (BehlerParrinello Gaussian symmetry)
                 + wACSF (weighted Gaussian symmetry)
-                + EAMD (embeded atom method descriptor)
+                + EAD (embeded atom density)
                 + SO4 (bispectrum)
                 + SO3 (smoothed powerspectrum)
             - Rc: float
@@ -37,13 +37,15 @@ class PyXtal_FF():
                 Compute rdxdr (needed for stress calculation) or not
             - force: bool (True)
                 Compute dxdr (needed for force calculation) or not
+             - cutoff: str
+                The cutoff function.
             - parameters: dict
                 Example,
                 + BehlerParrinello
-                    {'G2': {'eta': [1.3, 2.], 'Rs': [.1, .2]},
-                    'G4': {'eta': [.3, .7], 'lambda': [-1, 1], 'zeta': [.8, 2]}}
-                + SO4/Bispectrum
-                    {'L': 2, 'eta': [.3, .7], 'Rs': [.1, .2]}}
+                    {'G2': {'eta': [.3, 2.], 'Rs': [1., 2.]}, 
+                     'G4': {'eta': [.3, .7], 'lambda': [-1, 1], 'zeta': [.8, 2]}}
+                + EAD
+                    {'L': 2, 'eta': [.3, .7], 'Rs': [.1, .2]}
                 + SO4/Bispectrum
                     {'lmax': 3}
                 + SO3
@@ -128,7 +130,7 @@ class PyXtal_FF():
             self.print_logo()
         
         # Checking the keys in descriptors
-        descriptors_keywords = ['type', 'Rc', 'N_train', 'N_test', 
+        descriptors_keywords = ['type', 'Rc', 'N_train', 'N_test', 'cutoff',
                                 'force', 'stress', 'ncpu', 'parameters']
         if descriptors is not None:
             for key in descriptors.keys():
@@ -147,12 +149,13 @@ class PyXtal_FF():
                              'ncpu': 1,
                              'force': True,
                              'stress': True,
+                             'cutoff': 'cosine',
                              }
         
         # Update the default based on user-defined descriptors
         if descriptors is not None:
             self._descriptors.update(descriptors)
-            if 'type' in descriptors and descriptors['type'] == 'EAMD':
+            if 'type' in descriptors and descriptors['type'] == 'EAD':
                 _parameters = {'L': 3, 'eta': [0.1], 'Rs': [1.]}
             else:
                 _parameters = {'lmax': 3, 'rfac': 1.0, 'normalize_U': False}
@@ -382,7 +385,7 @@ class PyXtal_FF():
         """ Print the descriptors information. """
 
         print('Descriptor parameters:')
-        keys = ['type', 'Rc']
+        keys = ['type', 'Rc', 'cutoff']
         for key in keys:
             print('{:12s}: {:}'.format(key, _descriptors[key]))
 
@@ -390,7 +393,7 @@ class PyXtal_FF():
             key_params = ['lmax', 'normalize_U']
         elif _descriptors['type'] in ['SO3', 'SOAP']:
             key_params = ['nmax', 'lmax']
-        elif _descriptors['type'] == 'EAMD':
+        elif _descriptors['type'] == 'EAD':
             key_params = ['L', 'eta', 'Rs']
         else:
             key_params = []
